/*******************************************************************************
*   Author       -  Kiraly Tibor
*                   www.tkiraaly.hu
*   Date         -  2016.02.06.
*   Chip         -  Atmel ATtiny13a
*   Compiler     -  avr-gcc (WinAVR 20080610) 4.3.0
*
*   Definitions for ATtiny13a
*
********************************************************************************
*
*                                       +---------+
*   PCINT5 - dW - NRESET - ADC0 - PB5  ++ 1    8  |  +VCC
*          PCINT3 - CLKI - ADC3 - PB3  |  2    7  |  PB2 - ADC1 - SCK - T0 - PCINT2
*                 PCINT4 - ADC2 - PB4  |  3    6  |  PB1 - AIN1 - MISO - OC0B - INT0 - PCINT1
*                                 GND  |  4    5  |  PB0 - AIN0 - MOSI - OC0A - PCINT0
*                                      +----------+
*
********************************************************************************
*   PonyProg Configuration and Security Bits (P - bepipalva):
*
*               Default (by Manufacture)
*
*   LOCK2             - Memory protection
*   LOCK1
*
*   SELFPRGEN         - Self Programming Enable
*   DWEN              - debugWire Enable
*   BODLEVEL1         - Brown-out Detector trigger level
*   BODLEVEL0
*
*   SPIEN             - Enable Serial Programming and Data Downloading
*   EESAVE            - EEPROM memory is preserved through the Chip Erase
*   WDTON             - Watchdog Timer always on
*   CKDIV8      P     - Divide clock by 8
*   SUT1              - Select start-up time
*   SUT0        P
*   CKSEL1            - Select Clock source
*   CKSEL0      P
*
********************************************************************************
*   CKSEL1, CKSEL0
*   P       P       00  External Clock
*   P       -       01  Internal Oscillator 4,8 MHz
*   -       P       10  Internal Oscillator 9,6 MHz
*   -       -       11  Internal Oscillator 128 kHz
*
*   Calibrated Internal RC Oscillator
*   SUT1   SUT0
*   P      P        00  BOD enabled
*   P      -        01  Fast rising power
*   -      P        10  Slowly rising power
*   -      -        11  Reserved
*
*   LOCK2   LOCK1
*   -       -       11  No memory lock
*   -       P       10  Programming disable
*   P       P       11  Programming and verification disable
*
*   Brown-out Detector trigger level
*   BL1     BL0
*   P       P       11  4.3V
*   -       P       01  2.7V
*   P       -       10  1.8V
*   -       -       00  BOD disabled 
*
*******************************************************************************/

#ifndef tkiraaly_attiny13a
#define tkiraaly_attiny13a


#include <avr/sfr_defs.h>
#include "tkiraaly_avr_macros.h"


#define ADCSRB    _SFR_IO8(0x03)                 // ADC Control and Status Register
#define    ADTS0     0                           // ADC Auto Trigger Source
#define    ADTS1     1                           // ADC Auto Trigger Source
#define    ADTS2     2                           // ADC Auto Trigger Source
#define    ACME      6                           // Analog Comparator Multiplexer Enable

// ADTS2..0  Trigger source
//      000  Free Running mode
//      001  Analog Comparator
//      010  External Interrupt Request 0
//      011  Timer/Counter Compare Match A
//      100  Timer/Counter Overflow
//      101  Timer/Counter Compare Match B
//      110  Pin Change Interrupt Request

#define ADC       _SFR_IO16(0x04)                // ADC Data Register
#define ADCL      _SFR_IO8(0x04)                 // ADC Data Register L
#define ADCH      _SFR_IO8(0x05)                 // ADC Data Register H

#define ADCSRA    _SFR_IO8(0x06)                 // ADC Control and Status Register A
#define    ADPS0     0                           // ADC Prescaler Select Bits
#define    ADPS1     1                           // ADC Prescaler Select Bits
#define    ADPS2     2                           // ADC Prescaler Select Bits
#define    ADIE      3                           // ADC Interrupt Enable
#define    ADIF      4                           // ADC Interrupt Flag
#define    ADATE     5                           // ADC Auto Trigger Enable
#define    ADSC      6                           // ADC Start Conversion
#define    ADEN      7                           // ADC Enable
// Div  ADPS3..0
//   2  000
//   2  001
//   4  010
//   8  011
//  16  100
//  32  101
//  64  110
// 128  111


#define ADMUX     _SFR_IO8(0x07)                 // ADC Multiplexer Selection Register
#define    MUX0      0                           // Analog Channel Selection Bits
#define    MUX1      1                           // Analog Channel Selection Bits
#define    ADLAR     5                           // ADC Left Adjust Result
#define    REFS0     6                           // Reference Selection Bit

// Input MUX1..0
// ADC0  00
// ADC1  01
// ADC2  10
// ADC3  11

#define ACSR      _SFR_IO8(0x08)                 // Analog Comparator Control and Status Register
#define    ACIS0     0                           // Analog Comparator Interrupt Mode Select
#define    ACIS1     1                           // Analog Comparator Interrupt Mode Select
#define    ACIE      3                           // Analog Comparator Interrupt Enable
#define    ACI       4                           // Analog Comparator Interrupt Flag
#define    ACO       5                           // Analog Comparator Output
#define    ACBG      6                           // Analog Comparator Bandgap Select
#define    ACD       7                           // Analog Comparator Disable

#define DIDR0     _SFR_IO8(0x14)                 // Digital Input Disable Register 0
#define    AIN0D     0
#define    AIN1D     1
#define    ADC1D     2
#define    ADC3D     3
#define    ADC2D     4
#define    ADC0D     5

#define PCMSK     _SFR_IO8(0x15)                 // Pin Change Mask Register
#define    PCINT0     0
#define    PCINT1     1
#define    PCINT2     2
#define    PCINT3     3
#define    PCINT4     4
#define    PCINT5     5

#define PINB      _SFR_IO8(0x16)                 // Port B - Input Register 0..5

#define DDRB      _SFR_IO8(0x17)                 // Port B - Data Direction Register

#define PORTB     _SFR_IO8(0x18)                 // Port B - Output Register

#define EECR      _SFR_IO8(0x1C)                 // EEPROM control register
#define    EERE      0                           // EEPROM Read Enable
#define    EEPE      1                           // EEPROM Program Enable
#define    EEMPE     2                           // EEPROM Master Program Enable
#define    EERIE     3                           // EEPROM Ready Interrupt Enable
#define    EEPM0     4                           // EEPROM Programming Mode Bits 0
#define    EEPM1     5                           // EEPROM Programming Mode Bits 1

#define EEDR      _SFR_IO8(0x1D)                 // EEPROM data register

#define EEARL     _SFR_IO8(0x1E)                 // EEPROM address register 

#define WDTCR     _SFR_IO8(0x21)                 // Watchdog Timer Control Register
#define     WDP0     0                           // Watchdog Timer Prescaler 0
#define     WDP1     1                           // Watchdog Timer Prescaler 1
#define     WDP2     2                           // Watchdog Timer Prescaler 2
#define     WDE      3                           // Watchdog System Reset Enable
#define     WDCE     4                           // Watchdog Change Enable
#define     WDP3     5                           // Watchdog Timer Prescaler 3
#define     WDTIE    6                           // Watchdog Timer Interrupt Enable
#define     WDTIF    7                           // Watchdog Timer Interrupt Flag
// Time   Cycles  WDP3..0
// 16ms       2k  0000
// 32ms       4k  0001
// 64ms       8k  0010
// 0.125s    16k  0011
// 0.25s     32k  0100
// 0.5s      64k  0101
// 1s       128k  0110
// 2s       256k  0111
// 4s       512k  1000
// 8s      1024k  1001

#define PRR       _SFR_IO8(0x25)                 // Power Reduction Register
#define    PRADC     0                           // Power Reduction ADC
#define    PRTIM0    1                           // Power Reduction Timer/Counter0

#define CLKPR     _SFR_IO8(0x26)                 // Clock Prescale Register
#define    CLKPS0    0                           // Clock Prescaler Select Bits 0
#define    CLKPS1    1                           // Clock Prescaler Select Bits 1
#define    CLKPS2    2                           // Clock Prescaler Select Bits 2
#define    CLKPS3    3                           // Clock Prescaler Select Bits 3
#define    CLKPCE    7                           // Clock Prescaler Change Enable
// Div - CLKPS3..0
//   1 - 0000 - default
//   2 - 0001
//   4 - 0010
//   8 - 0011
//  16 - 0100
//  32 - 0101
//  64 - 0110
// 128 - 0111
// 256 - 1000

#define GTCCR     _SFR_IO8(0x28)                 // General Timer/Counter Control Register
#define    PSR10     0                           // Prescaler Reset Timer/Counter
#define    TSM       7                           // Timer/Counter Synchronization Mode

#define OCR0B     _SFR_IO8(0x29)                 // Timer/Counter – Output Compare Register B

#define DWDR      _SFR_IO8(0x2E)                 // debugWire Data Register

#define TCCR0A    _SFR_IO8(0x2F)                 // Timer/Counter Control Register A
#define    WGM00     0                           // Waveform Generation Mode 0
#define    WGM01     1                           // Waveform Generation Mode 1
#define    COM0B0    4                           // Compare Match Output B Mode 0
#define    COM0B1    5                           // Compare Match Output B Mode 1
#define    COM0A0    6                           // Compare Match Output A Mode 0
#define    COM0A1    7                           // Compare Match Output A Mode 1

#define BODCR     _SFR_IO8(0x30)                 // Brown-Out Detector Control Register
#define    BPDSE     0                           // BOD Sleep Enable
#define    BPDS      1                           // BOD Sleep

#define OSCCAL    _SFR_IO8(0x31)                 // Oscillator Calibration Register 

#define TCNT0     _SFR_IO8(0x32)                 // Timer/Counter (8-bit)

#define TCCR0B    _SFR_IO8(0x33)                 // Timer/Counter Control Register B
#define    CS00      0                           // Clock Select 0
#define    CS01      1                           // Clock Select 1
#define    CS02      2                           // Clock Select 2
#define    WGM02     3                           // Waveform Generation Mode
#define    FOC0B     6                           // Force Output Compare B
#define    FOC0A     7                           // Force Output Compare A
// CS02..0   description
//     000  - 0 - Stop
//     001  - F_CPU/1
//     010  - F_CPU/8
//     011  - F_CPU/64
//     100  - F_CPU/256
//     101  - F_CPU/1024
//     110  - External clock source on T0 pin. Clock on falling edge.
//     111  - External clock source on T0 pin. Clock on rising edge.

#define MCUSR     _SFR_IO8(0x34)                 // MCU Status Register
#define    PORF      0                           // Power-on Reset Flag
#define    EXTRF     1                           // External Reset Flag
#define    BORF      2                           // Brown-out Reset Flag
#define    WDRF      3                           // Watchdog Reset Flag

#define MCUCR     _SFR_IO8(0x35)                 // MCU Control Register
#define    ISC00     0                           // Interrupt Sense Control 0
#define    ISC01     1                           // Interrupt Sense Control 1
#define    SM0       3                           // Sleep Mode Select Bits 
#define    SM1       4                           // Sleep Mode Select Bits
#define    SE        5                           // Sleep Enable 
#define    PUD       6                           // Pull Up Resistors Disable

#define OCR0A     _SFR_IO8(0x36)                 // Timer/Counter – Output Compare Register A

#define SPMCSR    _SFR_IO8(0x37)                 // Store Program Memory Control and Status Register
#define    SPMEN     0                           // Self Programming Enable
#define    PGERS     1                           // Page Erase
#define    PGWRT     2                           // Page Write
#define    RFLB      3                           // Read Fuse and Lock Bits
#define    CTPB      4                           // Clear Temporary Page Buffer

#define TIFR0     _SFR_IO8(0x38)                 // Timer/Counter Interrupt Flag Register
#define    TOV0      1                           // Timer/Counter Overflow Flag
#define    OCF0A     2                           // Output Compare Flag A
#define    OCF0B     3                           // Output Compare Flag B

#define TIMSK0    _SFR_IO8(0x39)                 // Timer/Counter Interrupt Mask Register 0
#define    TOIE0     1                           // Timer/Counter0 Overflow Interrupt Enable
#define    OCIE0A    2                           // Timer/Counter0 Output Compare Match A Interrupt Enable
#define    OCIE0B    3                           // Timer/Counter Output Compare Match B Interrupt Enable

#define GIFR      _SFR_IO8(0x3A)                 // General Interrupt Flag Register
#define    PCIF      5                           // Pin Change Interrupt Flag
#define    INTF0     6                           // External Interrupt Flag 0

#define GIMSK     _SFR_IO8(0x3B)                 // General Interrupt Mask Register
#define    PCIE      5                           // Pin Change Interrupt Enable
#define    INT0      6                           // External Interrupt Request 0 Enable


/* Constants */
#define SPM_PAGESIZE (32)
#define RAMSTART     (0x60)
#define RAMSIZE      (64)
#define RAMEND       (RAMSTART + RAMSIZE - 1)
#define XRAMSTART    (NA)
#define XRAMSIZE     (0)
#define XRAMEND      RAMEND
#define E2END        (64 - 1)
#define E2PAGESIZE   (4)
#define FLASHEND     (1024 - 1)


/* Fuses */
#define FUSE_MEMORY_SIZE 2


/* Low Fuse Byte */
#define FUSE_CKSEL0 (unsigned char)~_BV(0)      // Select Clock Source
#define FUSE_CKSEL1 (unsigned char)~_BV(1)      // Select Clock Source
#define FUSE_SUT0   (unsigned char)~_BV(2)      // Select start-up time
#define FUSE_SUT1   (unsigned char)~_BV(3)      // Select start-up time
#define FUSE_CKDIV8 (unsigned char)~_BV(4)      // Start up with system clock divided by 8
#define FUSE_WDTON  (unsigned char)~_BV(5)      // Watch dog timer always on
#define FUSE_EESAVE (unsigned char)~_BV(6)      // Keep EEprom contents during chip erase
#define FUSE_SPIEN  (unsigned char)~_BV(7)      // SPI programming enable
#define LFUSE_DEFAULT (FUSE_SPIEN & FUSE_CKDIV8 & FUSE_SUT0 & FUSE_CKSEL0)


/* High Fuse Byte */
#define FUSE_RSTDISBL  (unsigned char)~_BV(0)   // Disable external reset
#define FUSE_BODLEVEL0 (unsigned char)~_BV(1)   // Enable BOD and select level
#define FUSE_BODLEVEL1 (unsigned char)~_BV(2)   // Enable BOD and select level
#define FUSE_DWEN      (unsigned char)~_BV(3)   // DebugWire Enable
#define FUSE_SELFPRGEN (unsigned char)~_BV(4)   // Self Programming Enable
#define HFUSE_DEFAULT  (0xFF)


/* Lock Bits */
#define __LOCK_BITS_EXIST


/* Signature */
#define SIGNATURE_0 0x1E
#define SIGNATURE_1 0x90
#define SIGNATURE_2 0x07




// KT

#define PULL_UP_R_DISABLE    B_1( MCUCR, PUD)                   // felhuzo ellenallasok tiltasa




// Interrupts

#define IT_INT0           _VECTOR( 1)  // External Interrupt
#define IT_INT0_ENABLE           B_1(GIMSK,INT0)
#define IT_INT0_DISABLE          B_0(GIMSK,INT0)
#define IT_INT0_LEVEL_0          MCUCR&= 0B11111100
#define IT_INT0_FALLING          MCUCR&= 0B11111100; MCUCR|= 0B00000010
#define IT_INT0_RISING                               MCUCR|= 0B00000011

#define IT_PCINT0         _VECTOR( 2)  // Pin Change Interrupt
#define IT_PCINT0_ENABLE         B_1(GIMSK,PCIE)
#define IT_PCINT0_DISABLE        B_0(GIMSK,PCIE)

#define IT_TC0_OVF        _VECTOR( 3)  // Timer/Counter Overflow
#define IT_TC0_OVF_ENABLE        B_1(TIMSK0,TOIE0)
#define IT_TC0_OVF_DISABLE       B_0(TIMSK0,TOIE0)

#define IT_EE_RDY         _VECTOR( 4) // EEPROM Ready
#define IT_EE_RDY_ENABLE         B_1(EECR,EERIE)
#define IT_EE_RDY_DISABLE        B_0(EECR,EERIE)

#define IT_ACOMP          _VECTOR( 5) // Analog Comparator
#define IT_ACOMP_ENABLE          B_1(ACSR,ACIE)
#define IT_ACOMP_DISABLE         B_0(ACSR,ACIE)

#define IT_TC0_COMPA      _VECTOR( 6)  // Timer/Counter Compare Match A
#define IT_TC0_COMPA_ENABLE      B_1(TIMSK0,OCIE1A)
#define IT_TC0_COMPA_DISABLE     B_0(TIMSK0,OCIE1A)

#define IT_TC0_COMPB      _VECTOR( 7)  // Timer/Counter Compare Match B
#define IT_TC0_COMPB_ENABLE      B_1(TIMSK0,OCIE1B)
#define IT_TC0_COMPB_DISABLE     B_0(TIMSK0,OCIE1B)

#define IT_WDT            _VECTOR( 8)  // Watchdog Time-out
#define IT_WDT_ENABLE            B_1(WDTCR,WDTIE)
#define IT_WDT_DISABLE           B_0(WDTCR,WDTIE)

#define IT_ADC            _VECTOR( 14) // ADC Conversion Complete
#define IT_ADC_ENABLE            B_1(ADCSRA,ADIE)
#define IT_ADC_DISABLE           B_0(ADCSRA,ADIE)

#define ISR(vector) void vector (void) __attribute__ ((signal,used)); void vector (void)
#define _VECTOR_SIZE 2                 // Size of individual vector
#define _VECTORS_SIZE (10 * _VECTOR_SIZE)




// ADC-hez

// ADC orajel elooszto valaszasa, lehetoleg nagy orajelhez
#ifndef ADC_PRESC
   #if F_CPU > 12 MHZ                      // div 128
      #define                ADC_PRESC  0B00000111
   #else
      #if F_CPU > 6 MHZ                    // div 64
         #define             ADC_PRESC  0B00000110
      #else
         #if F_CPU > 3 MHZ                 // div 32
            #define          ADC_PRESC  0B00000101
         #else
            #if F_CPU > 1,5 MHZ             // div 16
               #define       ADC_PRESC  0B00000100 
            #else
               #if F_CPU > 0,75 MHZ         // div 8
                  #define    ADC_PRESC  0B00000011 
               #else
                  #if F_CPU > 0,37 MHZ      // div 4
                     #define ADC_PRESC  0B00000010 
                  #else                     // div 2
                     #define ADC_PRESC  0B00000001 
                  #endif
               #endif
            #endif
         #endif
      #endif
   #endif
#endif

#define ADC_8BIT             B_1( ADMUX, ADLAR)                      // 8 bit felbontas, ADCH
#define ADC_10BIT            B_0( ADMUX, ADLAR)                      // 10 bit felbontas, ADCH+ ADCL

#define ADC_VREF_AVCC        B_0( ADMUX, REFS0)                      // Uref - AVCC lab
#define ADC_VREF_INT         B_0( ADMUX, REFS0)                      // Uref - belso 1V1

#define ADC_IN0              ADMUX&= 0B11111100                      // bemenet valasztas
#define ADC_IN1              ADMUX&= 0B11111100; ADMUX|= 0B00000001
#define ADC_IN2              ADMUX&= 0B11111100; ADMUX|= 0B00000010
#define ADC_IN3              ADMUX&= 0B11111100; ADMUX|= 0B00000011

#define ADC_FREE_RUN         ADCSRA= 0B11100000+ ADC_PRESC           // ADC folyamatos
#define ADC_START            ADCSRA= 0B11001000+ ADC_PRESC           // ADC inditas IT-hez

#define ADC_DISABLE          B_0( ADCSRA, ADEN)                      // ADC tiltasa
#define ADC_ENABLE           B_1( ADCSRA, ADEN)                      // ADC tiltas feloldasa




// TC0 szamlalohoz - 8 bit
// Regiszterek - TCNT0, TCCR0A, TCCR0B, OCR0A, OCR0B, TIMSK0, TIFR0  
// OCR0= TOP
// IT_TC0_OVF, amikor TCNT0 255-rol 0-ra ugrik.
// IT_TC0_COMP, amikor TCNT0 == OCR0 
// Szamlalo ujraallitasa elott TCCR1A-t es TCCR1B-t torolni kell!
// TIMER - ido_usec= (OCR0A- 1/ ( F_CPU/ osztas))* 1000000
// CLOCK - f_hz= ( F_CPU/ osztas)/ ( OCR0A+ 1)
// WAVE  - f_hz= F_CPU / ( 2* osztas *( OCR0_+ 1)
// PWM   - f_hz= F_CPU/ osztas/ 256

// TCR0B  TCR0A  TCR0A
// WGM02  WGM01  WGM00
//     0      0      0 - Normal
//     0      1      0 - CTC
//     0      1      1 - Fast PWM

//  TCR0A  TCR0A
// COM0_1 COM0_0
//      0      0 - NON PWM DISCONNECT
//      0      1 - NON PWM TOGGLE
//      1      1 - PWM

#define TC0_RESET             TCNT0= 0
#define TC0_DIVIDER_RESET     B_1(GTCCR,PSR10)
#define TC0_STOP              TCCR0B&= 0B11111000
#define TC0_F_1               TCCR0B&= 0B11111000; TCCR0B|= 0B00000001    // F_CPU/ 1
#define TC0_F_8               TCCR0B&= 0B11111000; TCCR0B|= 0B00000010    // F_CPU/ 8
#define TC0_F_64              TCCR0B&= 0B11111000; TCCR0B|= 0B00000011    // F_CPU/ 64
#define TC0_F_256             TCCR0B&= 0B11111000; TCCR0B|= 0B00000100    // F_CPU/ 256
#define TC0_F_1024            TCCR0B&= 0B11111000; TCCR0B|= 0B00000101    // F_CPU/ 1024
#define TC0_EXT_FALLING       TCCR0B&= 0B11111000; TCCR0B|= 0B00000110    // kulso orajel (T0) lefuto elere
#define TC0_EXT_RISING                             TCCR0B|= 0B00000111    // kulso orajel (T0) felfuto elere
#define TC0_CTC               B_1( TCR0A, WGM01)
#define TC0_PWM               B_1( TCR0A, WGM01); B_1( TCR0A, WGM00)

// szamol
#define TC0_COUNT             ""

// OCR0A ertekenel IT_TC0_COMP-t aktivalja
#define TC0_TIMER                         IT_TC0_COMPA_ENABLE
#define TC0_TIMER_OCR( ido_usec, osztas)  OCR0A= OCR_TIMER( ido_usec, osztas)  

// OCR0A ertekenel szamlalot nullazza es IT_TC0_COMP-t aktivalja
#define TC0_CLOCK                         TC0_CTC; IT_TC0_COMPA_ENABLE 
#define TC0_CLOCK_OCR( f_hz, osztas)      OCR0A= OCR_CLOCK( f_hz, osztas)   

// OCR0A ertekenel szamlalot nullazza es OC0A labat invertalja                                 
#define TC0_WAVE_A                        TC0_CTC; B_1(TCCR0A,COM0A0); PB0_OUT
#define TC0_WAVE_A_OCR( f_hz, osztas)     OCR0A= OCR_WAVE( f_hz, osztas) 

// OCR0B ertekenel szamlalot nullazza es OC0B labat invertalja                                 
#define TC0_WAVE_B                        TC0_CTC; B_1(TCCR0A,COM0B0); PB1_OUT 
#define TC0_WAVE_B_OCR( f_hz, osztas)     OCR0B= OCR_WAVE( f_hz, osztas)   

// OCR0A ertekenel OC0A lab= 1, majd 0xFF erteknel OC0A lab= 0 
#define TC0_PWM_A                         TC0_PWM; B_1(TCCR0A,COM0A1); B_1(TCCR0A,COM0A0); PB0_OUT

// OCR0B ertekenel OC0B lab= 1, majd 0xFF erteknel OC0B lab= 0 
#define TC0_PWM_B                         TC0_PWM; B_1(TCCR0A,COM0B1); B_1(TCCR0A,COM0B0); PB1_OUT


#endif